

def mlflow_get_models_by_interval(stage :str = "Staging", interval :str = "1h", mlflow_connection=None):

    models = mlflow_connection.search_registered_models()
    models_interval = []
    for model in models:
        tag = model.tags.get(stage)
        interval_model = tag.split('/')[1]

        if interval == interval_model:
            models_interval.append(model)

    return models_interval

def get_intervals_from_models(stage :str = "Staging", mlflow_connection=None):
    
    models = mlflow_connection.search_registered_models()
    intervals = []
    for model in models:
        tag = model.tags.get(stage)
        interval = tag.split('/')[1]
        intervals.append(interval)

    intervals = list(set(intervals))

    return intervals



from dotenv import dotenv_values
import mlflow
config = dotenv_values(".env")
import uuid
import json 
from mlflow.tracking import MlflowClient

# from tests.functions.create_experiment import create_experiment

id_test = str(uuid.uuid4()).replace('-','')[-10:-1]



def test_generate_list_of_experiments(pass_test: str = False):

    import pyttrading as pytrade
    experiments = pytrade.models.ExperimentMessage()
    experiment_data = pytrade.generate_experiments_list(experiments)
    if len(experiment_data['combinations']) > 0:
        pass_test = True 
    assert pass_test


def create_experiment():

    import pyttrading as pytrade

    id_test = str(uuid.uuid4()).replace('-','')[-10:-1]

    experiments = pytrade.models.ExperimentMessage()

    if not experiments:
        raise ValueError("Not found the market data")

    experiments.experiment_name =  f'testing_{id_test}_' + experiments.experiment_name
    experiment_data = pytrade.generate_experiments_list(experiments)
    experiment_data['combinations'] = [experiment_data['combinations'][0]]

    # ..... 
    data = pytrade.get_market_data_from_playground(
        symbols=experiments.symbol, 
        interval=experiments.interval, 
        start_date='9/1/2023',
        end_date='1/5/2024',
        is_crypto=experiments.is_crypto
    )

    if len(data) == 0:
        raise ValueError("Not found the market data")
    
    mlflow_connection = pytrade.init_mlflow_connection()

    experiment_data.get('combinations')

    model_name = experiment_data.get('combinations')[0].get('strategy')
    child_name = experiment_data.get('combinations')[0].get('name')

    model = pytrade.ModelSelector(
                    model_name=model_name,
                    path_model="tmp",
                    type_model="basic",
                    configuration=experiments, 
                    df=data,
                    symbol=experiments.symbol,
                    interval=experiments.interval,
                    mlflow=mlflow_connection
                )
    

    stats_json, best_return, _, _, _, params= model.run_experiment_get_backtesting()
    parent_experiment = None
    experiment_id = None
    for child_name in ['1h_sma', '1h_rsi']:
        response = pytrade.mlflow_save(
            mlflow_instance=mlflow_connection,
            experiment_name=experiments.experiment_name,
            parent_name=experiment_data['parent'],
            child_name=child_name,
            metrics={
                "best_return": best_return, 
                **stats_json
            },
            parameters={
                "optimized_params": json.dumps(params),
                "symbol": experiments.symbol,
                "interval": experiments.interval
            },
            tags = {
                "method": params.get('method'),
                "strategy_name": params.get('strategy'),
                "strategy": params.get('strategy')
            },
            parent=parent_experiment,
            experiment_id=experiment_id
        )
        parent_experiment = response.get('parent_experiment')
        experiment_id = response.get('experiment_id')

    return response



id_test = str(uuid.uuid4()).replace('-','')[-10:-1]


def test_generate_list_of_experiments(pass_test: str = False):

    import pyttrading as pytrade


    experiments = pytrade.models.ExperimentMessage()
    experiment_data = pytrade.generate_experiments_list(experiments)
    if len(experiment_data['combinations']) > 0:
        pass_test = True 
    assert pass_test


def test_run_experiment_and_save_mlflow(pass_test: str = False):
    response = create_experiment()
    if response.get('child').get('run_id'):
        pass_test = True
        client = MlflowClient()
        client.delete_experiment(experiment_id=response.get('parent').get('experiment_id'))

    assert pass_test

def test_experiment_to_model(pass_test: str = False):

    import pyttrading as pytrade
    response = create_experiment()

    id_test = str(uuid.uuid4()).replace('-','')[-10:-1]
    experiments = pytrade.models.ExperimentMessage()
    best_run_id = response.get('child').get('run_id')

    model_uri = f"runs:/{best_run_id}/model"


    result = mlflow.register_model(
            model_uri=model_uri,
            name= 'testing_'+ experiments.symbol + '_' + experiments.tag + '_' + experiments.version + '_' + id_test,
            tags={
                "tags": json.dumps(response.get('tags')),
                "parameters": json.dumps(response.get('parameters')),
                "metrics": json.dumps(response.get('metrics'))
            }
        )
    
    client = MlflowClient()

    result_model = client.transition_model_version_stage(
            name=result.name,
            version=result.version,
            stage="Staging",
            archive_existing_versions=True
        )   
    
    if result_model.current_stage == 'Staging':
        pass_test = True
        client.delete_registered_model(name=result.name)
        client.delete_experiment(experiment_id=response.get('child').get('experiment_id'))

    assert pass_test


def test_run_sync_multi_experiment(pass_test: str = False):
    import pyttrading as pytrade

    experiment = pytrade.models.ExperimentMessage()
    mlflow_connection = pytrade.init_mlflow_connection()

    experiment.experiment_name = f"test_{id_test}_{experiment.experiment_name}"

    best_results, experiments_result = pytrade.execute_multi_experiment(experiment=experiment, debug_mode=True, mlflow_connection=mlflow_connection)

    if len(experiments_result) == 2:
        pass_test = True

    return pass_test


def test_run_sync_multi_experiment_nested(pass_test: str = False):

    import pyttrading as pytrade

    experiment = pytrade.models.ExperimentMessage()
    mlflow_connection = pytrade.init_mlflow_connection()

    experiment.experiment_name = f"test7_{experiment.experiment_name}"
    
    start_date_ny, end_date_ny, _ = pytrade.get_datetime_now_trade()
    experiment.start_date = start_date_ny
    experiment.end_date = end_date_ny
    best_results, experiments_result = pytrade.execute_multi_experiment(experiment=experiment, debug_mode=True, mlflow_connection=mlflow_connection)

    if len(experiments_result) == 2:
        pass_test = True

    return pass_test


from .opensearch_collector import OpensearchCollector
import os
from dotenv import dotenv_values

config = dotenv_values(".env")


# Old version -----
# def get_market_data_from_playground(
#         symbols: list = ['TNA', 'SPY'],
#         opensearch_host :str = os.getenv('URL_OPENSEARCH', 'http://localhost:9200'),
#         playground_host :str = os.getenv('URL_PLAYGROUND', 'http://localhost:5001'), 
#         start_date :str = '9/1/2023', 
#         end_date: str = '1/5/2024',
#         interval: str = "1h",
#         is_crypto: bool = False
#     ):

#     collector = OpensearchCollector(
#             url_opensearch=opensearch_host,
#             url_playground=playground_host,
#     )

#     data =  collector.market_data(
#             symbols=symbols,
#             start_date=start_date,
#             end_date=end_date,
#             interval=interval, 
#             is_crypto=is_crypto
#         )

#     return data


def get_market_data_from_playground(
        symbols: list = ['TNA', 'SPY'],
        opensearch_host :str = os.getenv('URL_OPENSEARCH', 'http://localhost:9200'),
        playground_host :str = os.getenv('URL_PLAYGROUND', 'http://localhost:5001'), 
        start_date :str = '9/1/2023', 
        end_date: str = '1/5/2024',
        interval: str = "1h",
        is_crypto: bool = False,
        use_playground: bool = False
    ):

    if use_playground:
        collector = OpensearchCollector(
                url_opensearch=opensearch_host,
                url_playground=playground_host,
        )

        data =  collector.market_data(
                symbols=symbols,
                start_date=start_date,
                end_date=end_date,
                interval=interval, 
                is_crypto=is_crypto
            )
    
    else:
        date_obj = datetime.strptime(start_date, '%m/%d/%Y')
        formatted_date = date_obj.strftime('%Y-%-m')            
        data = get_market_data_from_alpaca(
            symbols=symbols,
            api_key=config.get('ALPACA_API_KEY'),
            api_secret=config.get('ALPACA_API_SECRET'),
            start_date=str(formatted_date),
            interval=interval,
            is_crypto=is_crypto
        )

        data['date'] = data.index
        data.reset_index(drop=True, inplace=True)

    return data

from alpaca.data.historical import CryptoHistoricalDataClient
from alpaca.data.requests import CryptoBarsRequest, StockQuotesRequest, StockBarsRequest
from alpaca.data.timeframe import TimeFrame
from datetime import datetime
import pandas as pd
from alpaca.data import StockHistoricalDataClient, TimeFrame


def get_market_data_from_alpaca(
        symbols: list = ['SPY'],
        api_key :str = os.getenv('API_KEY'),
        api_secret :str = os.getenv('API_SECRET'), 
        start_date :str = "2023-11-19",
        interval: str = "1h",
        is_crypto: bool = False):
    
    start_time = pd.to_datetime(start_date).tz_localize('America/New_York')

    if "h" not in interval and  "d" not in interval: 
        raise ValueError("Interval invalid")

    if not is_crypto:

        data_client = StockHistoricalDataClient(api_key, api_secret)

        request_params = StockBarsRequest(
                symbol_or_symbols=symbols,
                timeframe=TimeFrame.Hour,
                start=start_time
        )

        data = data_client.get_stock_bars(request_params).df.tz_convert('America/New_York', level=1)

    else: 

        # client = CryptoHistoricalDataClient(api_key, api_secret)
        client = CryptoHistoricalDataClient(api_key, api_secret)

        request_params = CryptoBarsRequest(
                        symbol_or_symbols=symbols,
                        timeframe=TimeFrame.Hour,
                        start=start_time
                 )

                
        data = client.get_crypto_bars(request_params)
        data = data.df

    # Resampleamos los datos al intervalo especificado
    if interval.endswith('h') and interval != "1h":
        data.index = pd.to_datetime(data.index.get_level_values(1))
        interval_hours = int(interval[:-1])  # Extraemos la parte numérica del intervalo
        data = data.resample(f'{interval_hours}H').agg({
            'open': 'first',
            'high': 'max',
            'low': 'min',
            'close': 'last',
            'volume': 'sum'
        })
    else:
        data.index = pd.to_datetime(data.index.get_level_values(1))

    data['date'] = data.index
    data.reset_index(drop=True, inplace=True)
    return data


# open      high      low   close      volume
# timestamp                                                               
# 2024-02-29 04:00:00-05:00  505.20  505.4600  504.640  505.08    211743.0
# 2024-02-29 08:00:00-05:00  505.49  508.8455  504.500  506.65  23813465.0
# 2024-02-29 12:00:00-05:00  506.65  509.7400  506.505  508.00  42897475.0


from ..brokers import get_alpaca_strategy_profit
from ..models import StatusBot, TradingConfig
from ..brokers import AlpacaTrading
from ..utils import read_mlflow_model

def  get_status_bot(trading_config :TradingConfig, mlflow_connection=None):

    alpaca_trading = AlpacaTrading(trading_config.broker.api_key, trading_config.broker.api_secret)

    percentage, profit_percentage = get_alpaca_strategy_profit(trading_config)
    
    status_bot = StatusBot(
        orders_percentage=percentage,
        profit_history={
            "percentage": percentage,
            "profit_percentage": profit_percentage
        }, 
        replicate_latest_strategy_buy_action=True
    )

    if len(profit_percentage) == 0:
        status_bot.history_orders = False
    else:
        status_bot.history_orders = True

    if percentage > 0.0:
        status_bot.is_winner = True

    ## Get Registred Model data -----------------
    
    try:
        model_parameters = read_mlflow_model(
            symbol=trading_config.trading.symbol, 
            tag=trading_config.model.tag,
            stage=trading_config.model.stage, 
            mlflow_connection=mlflow_connection
        )

        if model_parameters: 
            status_bot.is_registred_model = True
            status_bot.model_parameters = model_parameters
    except Exception as e:
        print("Model not eixt")
        status_bot.is_registred_model = False
        status_bot.model_parameters = {}


    # Verify if exist the position
        
    
    positions = alpaca_trading.get_open_positions()
    for position in positions:
        if position.get('symbol') == trading_config.trading.symbol.replace('/', ''):
            status_bot.is_open_positions = True 

            market_value = float(position['market_value'])
            cost_basis = float(position['cost_basis'])
            profit_position = market_value - cost_basis
            status_bot.profit_position = profit_position


            profit_percentage_positions = (profit_position / cost_basis) * 100
            status_bot.profit_percentage_positions = profit_percentage_positions
            
            if profit_percentage_positions > 0.0:
                status_bot.is_winner = True

            break


    orders = alpaca_trading.get_orders_list_filter(status='accepted', symbol=trading_config.trading.symbol)
    for order in orders:
        if order.get('symbol') == trading_config.trading.symbol.replace('/', ''):
            status_bot.is_open_order = True

    return status_bot




import os

def test_get_market_data(pass_test :bool = False):

    import pyttrading as pytrade
    data = pytrade.get_market_data_from_playground(symbols=['TNA'], is_crypto=False)

    if len(data) > 200:
        pass_test = True
        
    assert pass_test
    
def test_get_market_crypto(pass_test :bool = False):

    import pyttrading as pytrade

    pass_test = False
    data = pytrade.get_market_data_from_playground(symbols=['AVAX/USD'], is_crypto=True)

    if len(data) > 200:
        pass_test = True

    assert pass_test


from dotenv import dotenv_values
config = dotenv_values(".env")


def test_get_market_data_alpaca(pass_test :bool = False):

    import pyttrading as tt
    api_key =  os.getenv('ALPACA_API_KEY', config.get('ALPACA_API_KEY')) 
    api_secret =  os.getenv('ALPACA_API_SECRET', config.get('ALPACA_API_SECRET')) 

    data = tt.get_market_data_from_alpaca(
        symbols=["SPY"],
        api_key=api_key,
        start_date = "2023-11-19",
        api_secret=api_secret,
        interval="4h",
        is_crypto=False
    )

    if len(data) > 100:
        pass_test = True 

    assert pass_test



def test_get_market_data_alpaca_crypto(pass_test :bool = False):

    import pyttrading as tt

    api_key =  os.getenv('ALPACA_API_KEY', config.get('ALPACA_API_KEY')) 
    api_secret =  os.getenv('ALPACA_API_SECRET', config.get('ALPACA_API_SECRET')) 

    data = tt.get_market_data_from_alpaca(
        symbols=["BTC/USD"],
        api_key=api_key,
        start_date = "2024-01-19",
        api_secret=api_secret,
        interval="4h",
        is_crypto=True
    )


    if len(data) > 100:
        pass_test = True 

    assert pass_test


from ..brokers import AlpacaTrading
import logging
from ..models import TradingConfig
log_format = '%(asctime)s| [%(levelname)s] %(message)s'
logging.basicConfig(level=logging.INFO, format=log_format)
log = logging.getLogger(__name__)



def get_alpaca_strategy_profit(trading_config :TradingConfig, limit :int = 20):

    alpaca_trading = AlpacaTrading(
        trading_config.broker.api_key, 
        trading_config.broker.api_secret
        )

    orders = alpaca_trading.get_orders_list_filter(
        symbol=trading_config.trading.symbol,
        limit=limit
    )
    
    found = None
    profit_list = []
    profit_percentage = []
    order_sell = None
    for order in orders:
        
        if order.get('side') == 'sell' and not found:
            found = True
            order_sell = order
        elif order.get('side') == 'buy' and order.get('status') != 'canceled' and order_sell:

            qty_sell = float(order_sell.get('filled_qty'))
            filled_avg_price_sell =  float(order_sell.get('filled_avg_price'))

            found = None
            filled_qty_buy = float(order.get('filled_qty'))
            filled_avg_price_buy = float(order.get('filled_avg_price'))

            national = float(order.get('notional'))

            # Calcula el costo total de compra y el ingreso total de venta
            national_buy = national
            national_sell = filled_avg_price_sell * qty_sell

            # Calcula el profit absoluto
            profit = national_sell - national_buy
            profit_list.append(profit)

            profit_p = 100 * (national_sell - national_buy)/national_buy
            profit_percentage.append(profit_p)

            log.info(f"PROFIT {trading_config.trading.symbol} {profit} {profit_p}%")

            order_sell = None


    percentage = sum(profit_percentage)

    return percentage, profit_percentage


# import pyttrading as pytrade
# from dotenv import dotenv_values
# config = dotenv_values(".env")

# def run_experiment(strategy :str = 'rsi'):

#     experiment = pytrade.ExperimentMessage(
#         symbol='TNA',
#         intervals=[
#             '1h'
#         ],
#         strategies_list=[
#             strategy,
#         ],
#         is_crypto=False
#     )

#     data = pytrade.get_market_data_from_playground(
#         symbols=experiment.symbol, 
#         interval=experiment.interval, 
#         start_date='9/1/2023',
#         end_date='1/5/2024',
#         is_crypto=experiment.is_crypto
#     )

#     if len(data) == 0:
#         raise ValueError("Not found the market data")
    
#     strategies_name = strategy

#     mlflow_connection = pytrade.init_mlflow_connection()

#     model = pytrade.ModelSelector(
#                     model_name=strategies_name,
#                     path_model="tmp",
#                     type_model="basic",
#                     configuration=experiment, 
#                     df=data,
#                     symbol=experiment.symbol,
#                     interval=experiment.intervals[0],
#                     mlflow=mlflow_connection
#                 )
    

#     _, best_return, _, _, _, _= model.run_experiment_get_backtesting()
   
#     return best_return



import logging

log_format = '%(asctime)s| [%(levelname)s] %(message)s'
# logging.basicConfig(level=logging.INFO, format=log_format)
logging.basicConfig(level=logging.DEBUG, format=log_format)

log = logging.getLogger(__name__)






def run_experiment(strategy :str = 'rsi'):

    import pyttrading as pytrade
    from dotenv import dotenv_values
    config = dotenv_values(".env")

    experiment = pytrade.ExperimentMessage(
        symbol='TNA',
        intervals=[
            '1h'
        ],
        strategies_list=[
            strategy,
        ],
        is_crypto=False
    )

    data = pytrade.get_market_data_from_playground(
        symbols=experiment.symbol, 
        interval=experiment.interval, 
        start_date='9/1/2023',
        end_date='1/5/2024',
        is_crypto=experiment.is_crypto
    )

    if len(data) == 0:
        raise ValueError("Not found the market data")
    
    strategies_name = strategy

    mlflow_connection = pytrade.init_mlflow_connection()

    model = pytrade.ModelSelector(
                    model_name=strategies_name,
                    path_model="tmp",
                    type_model="basic",
                    configuration=experiment, 
                    df=data,
                    symbol=experiment.symbol,
                    interval=experiment.intervals[0],
                    mlflow=mlflow_connection
                )
    

    _, best_return, _, _, _, _= model.run_experiment_get_backtesting()
   
    return best_return


from dotenv import dotenv_values
config = dotenv_values(".env")

def test_experiment_rsi(pass_test: str = False):
    best_return = run_experiment(strategy= 'rsi')
    if best_return > 1.5:
        pass_test = True 
    assert pass_test
    

def test_experiment_sma(pass_test: str = False):
    best_return = run_experiment(strategy= 'sma')
    if best_return > 0.5:
        pass_test = True 
    assert pass_test

def test_experiment_ema(pass_test: str = False):
    best_return = run_experiment(strategy= 'ema')
    if best_return > 1.05:
        pass_test = True 
    assert pass_test

def test_experiment_dummy(pass_test: str = False):
    best_return = run_experiment(strategy='dummy')
    if best_return > -10:
        pass_test = True 
    assert pass_test

def test_experiment_macd(pass_test: str = False):
    best_return = run_experiment(strategy='macd')
    if best_return > 0.1:
        pass_test = True 
    assert pass_test

def test_experiment_breakout(pass_test: str = False):
    best_return = run_experiment(strategy='breakout')
    if best_return > 0.0:
        pass_test = True 
    assert pass_test




from . import AlpacaTrading
from ..models.trading import Trading

# from alpaca.trading.client import TradingClient

# import alpaca.trading as alpaca_trading



def alpaca_open_long(trading :Trading, api_key, api_secret, tag :str ="algo"):

    strategy_name = f"{trading.symbol}_{tag}"

    if  trading.time_in_force == "":
        if trading.is_crypto: 
            trading.time_in_force = 'gtc'
        else: 
            trading.time_in_force = 'day'

    alpaca_trading = AlpacaTrading(api_key, api_secret)

    client_order_id = alpaca_trading.open_position(
        symbol=trading.symbol, 
        quantity=trading.notional,
        stop_loss=trading.stop_loss, 
        strategy_name=strategy_name, 
        return_client_order_id=True, 
        time_in_force=trading.time_in_force
    )
    
    return client_order_id


def alpaca_close_long(symbol, api_key, api_secret, tag="test"):


    alpaca_trading = AlpacaTrading(api_key, api_secret)

    strategy_name = f"{symbol}_{tag}"

    client_order_id = alpaca_trading.close_position(
        symbol=symbol, 
        strategy_name=strategy_name
    )

    return client_order_id



from pydantic import BaseModel


class Broker(BaseModel):
    id :str = ""
    provider :str = "alpaca"
    api_key :str = "******"
    api_secret :str = "***********"

class Owner(BaseModel):
    username :str = "username"
    surname :str = "surname"
    id :str = "00000001"

class Symbol(BaseModel):
    symbol: str = "TNA"
    is_crypto: bool = False

class Model(BaseModel):
    tag: str = "algo"
    stage: str = "Staging"
    version: str = "0.0.1"
    provider :str = "mlflow"

class Trading(Symbol):
    id :str = "123123123123"
    notional: float = 200.0
    side :str = 'buy'
    type :str = 'market'
    time_in_force :str = 'market'
    stop_loss :float = 100.0
    client_order_id :str = 'Client Id'
    is_paper :bool = True

class TradingConfig(BaseModel):
    id :str = "123123123123"
    active :bool = True 
    replicate_latest_open_long_signal :bool = False 
    calculate_best_stop_loss :bool = True
    broker :Broker
    trading :Trading 
    owner :Owner
    model :Model = Model()

class StatusBot(BaseModel):

    # Controller Variables -----
    is_open_order :bool = False
    is_registred_model :bool = False 
    is_open_positions :bool = False
    is_winner :bool = False

    # DataSaved 
    model_parameters :dict = {}

    # Controller Value
    profit_factor_limit :float = 1.2



    new_strategy :bool = False
    history_orders: bool = False


    profit_history :dict = {}
    best_experiment :float = 0.0
    orders_percentage :float = 0.0

    replicate_latest_strategy_buy_action :bool = False
    profit_percentage_positions :float = 0.0
    profit_position :float = 0.0

    experiment_profit_factor :float = 0.0
    experiment_profit_factor_sl :float = 0.0


    
    experiment_best_return :float = 0.0
    experiment_best_return_stop_loss :float = 0.0
    experiment_best_stop_loss :float = 0.0
    experiment_return :float = 0.0




# from ... import pyttrading as pytrade
# from dotenv import dotenv_values
# config = dotenv_values(".env")


# import uuid
# import json 

# id_test = str(uuid.uuid4()).replace('-','')[-10:-1]

# def test_generate_list_of_experiments(pass_test: str = False):
#     experiments = pytrade.models.ExperimentMessage()
#     experiment_data = pytrade.generate_experiments_list(experiments)
#     if len(experiment_data['combinations']) > 0:
#         pass_test = True 
#     assert pass_test


# def create_experiment():

#     id_test = str(uuid.uuid4()).replace('-','')[-10:-1]

#     experiments = pytrade.models.ExperimentMessage()

#     if not experiments:
#         raise ValueError("Not found the market data")

#     experiments.experiment_name =  f'testing_{id_test}_' + experiments.experiment_name
#     experiment_data = pytrade.generate_experiments_list(experiments)
#     experiment_data['combinations'] = [experiment_data['combinations'][0]]

#     # ..... 
#     data = pytrade.get_market_data_from_playground(
#         symbols=experiments.symbol, 
#         interval=experiments.interval, 
#         start_date='9/1/2023',
#         end_date='1/5/2024',
#         is_crypto=experiments.is_crypto
#     )

#     if len(data) == 0:
#         raise ValueError("Not found the market data")
    
#     mlflow_connection = pytrade.init_mlflow_connection()

#     experiment_data.get('combinations')

#     model_name = experiment_data.get('combinations')[0].get('strategy')
#     child_name = experiment_data.get('combinations')[0].get('name')

#     model = pytrade.ModelSelector(
#                     model_name=model_name,
#                     path_model="tmp",
#                     type_model="basic",
#                     configuration=experiments, 
#                     df=data,
#                     symbol=experiments.symbol,
#                     interval=experiments.interval,
#                     mlflow=mlflow_connection
#                 )
    

#     stats_json, best_return, _, _, _, params= model.run_experiment_get_backtesting()
#     parent_experiment = None
#     experiment_id = None
#     for child_name in ['1h_sma', '1h_rsi']:
#         response = pytrade.mlflow_save(
#             mlflow_instance=mlflow_connection,
#             experiment_name=experiments.experiment_name,
#             parent_name=experiment_data['parent'],
#             child_name=child_name,
#             metrics={
#                 "best_return": best_return, 
#                 **stats_json
#             },
#             parameters={
#                 "optimized_params": json.dumps(params),
#                 "symbol": experiments.symbol,
#                 "interval": experiments.interval
#             },
#             tags = {
#                 "method": params.get('method'),
#                 "strategy_name": params.get('strategy'),
#                 "strategy": params.get('strategy')
#             },
#             parent=parent_experiment,
#             experiment_id=experiment_id
#         )
#         parent_experiment = response.get('parent_experiment')
#         experiment_id = response.get('experiment_id')

#     return response


import json
from .logs import log
from .get_datetime_now_trade import get_datetime_now_trade
from ..market_data import get_market_data_from_playground
from ..strategies import ModelSelector
from . import init_mlflow_connection

import uuid
id_test = str(uuid.uuid4()).replace('-','')[-10:-1]

import mlflow


def eval_strategy_latest(parameters=None, get_action_latest :bool = False):

    symbol = parameters.get('symbol')
    interval = parameters.get('interval')
    is_crypto = parameters.get('is_crypto')
    start_date, end_date, now_ny = get_datetime_now_trade()
    optimized_params = json.loads(parameters.get('optimized_params'))
    parms = optimized_params.get('params')

    data = get_market_data_from_playground(
        symbols=symbol, 
        interval=interval, 
        start_date=str(start_date),
        end_date=str(end_date),
        is_crypto=is_crypto
    )

    if len(data) == 0:
        raise ImportError("Not found data")
    

    model = ModelSelector(
                    model_name=optimized_params.get("strategy"),
                    path_model="tmp",
                    type_model="basic",
                    configuration=parameters, 
                    df=data,
                    symbol=symbol,
                    interval=interval,
                    mlflow=mlflow
                )
    
    strategy_model = model.basic_models()
    strategy = strategy_model()

    df_action = strategy.eval(df=data, params=parms)

    if not get_action_latest:
        actions = df_action['actions'].iloc[-1]
    else:
        actions = [accion for accion in df_action['actions'].iloc[-100:-1] if accion != 0]
        actions = actions[-1]


    action = None
    if actions == 1:
        action = 'open-long'
        message = f"🟢 BUY: {action}"
    elif actions == 2:
        action = 'close-long'
        message = f"🟠 SELL: {action}"
    elif actions == 0:
        action = 'keep'
        message = f"🔵 KEEP: {action}"

    close_price = round(df_action['close'].iloc[-1],2)
    log.info(f"{message} [{interval}] {parameters} START: {start_date} END:{end_date}  CLOSE: {close_price}")

    return action


def read_mlflow_model(symbol, tag, mlflow_connection, stage):
    model_name = f"{symbol}_{tag}"

    log.info(f"Starting Eval model: {model_name}")

    results = mlflow_connection.search_registered_models(filter_string=f"name='{model_name}'")
    model_selected = None
    if len(results) == 0:
        raise AssertionError(f"Not found model, {symbol}/{stage}/{tag}")
    
    for model in results[0].latest_versions:
        current_stage = model.current_stage
        if current_stage == stage:
            model_selected = model 
            break

    tags_selected = model_selected.tags
    if model_selected:
        parameters = json.loads(tags_selected.get('parameters'))
        return parameters
    
    return False


def eval_mlflow_model_strategy(symbol :str, stage :str, tag: str, mlflow_connection=None, get_action_latest :bool = False):

    model_name = f"{symbol}_{tag}"

    log.info(f"Starting Eval model: {model_name}")

    results = mlflow_connection.search_registered_models(filter_string=f"name='{model_name}'")
    model_selected = None
    if len(results) == 0:
        raise AssertionError(f"Not found model, {symbol}/{stage}/{tag}")
    
    for model in results[0].latest_versions:
        current_stage = model.current_stage
        if current_stage == stage:
            model_selected = model 
            break

    tags_selected = model_selected.tags
    if model_selected:
        parameters = json.loads(tags_selected.get('parameters'))
        action = eval_strategy_latest(parameters, get_action_latest)

    return action


def get_model_tags(symbol :str, stage :str, tag: str, mlflow_connection=None):

    model_name = f"{symbol}_{tag}"

    log.info(f"Starting Eval model: {model_name}")

    results = mlflow_connection.search_registered_models(filter_string=f"name='{model_name}'")
    model_selected = None
    if len(results) == 0:
        raise AssertionError(f"Not found model, {symbol}/{stage}/{tag}")
    
    for model in results[0].latest_versions:
        current_stage = model.current_stage
        if current_stage == stage:
            model_selected = model 
            break

    tags_selected = model_selected.tags

    return tags_selected


# Xcaret
# Nombre de origen azteca que significa "la que escucha"
from ..models import TradingConfig, Broker, ExperimentMessage, get_datetime_now_trade
from .status_bot import get_status_bot
from ..utils.logs import log
from ..experiments import execute_multi_experiment
from ..stop_loss import optimize_stop_loss_simple
from ..models import save_best_model_mlflow
from ..brokers import alpaca_open_long, alpaca_close_long
from ..utils.eval_mlflow_model_strategy import eval_mlflow_model_strategy

class BotXcaret:
    # TODO 

    def __init__(self, trading_config :TradingConfig, debug_mode :bool = False, mlflow_connection=None):
        self.trading_config = trading_config

        #! True: Run only two experiments
        self.debug_mode = debug_mode
        self.mlflow_connection = mlflow_connection
      
        self.min_profit_factor_allowed = 1.2
    

    def get_status(self):

        status_bot  = get_status_bot(
            trading_config=self.trading_config,
            mlflow_connection=self.mlflow_connection
            )
        
        self.status_bot = status_bot
        return status_bot

    
    def run_mlflow_experiments(self):

        start_date_ny, end_date_ny, _ = get_datetime_now_trade()

        experiment_name = f"{self.trading_config.trading.symbol}_{self.trading_config.model.tag}_{self.trading_config.model.version}"
        self.experiment_name = experiment_name

        experiments= ExperimentMessage(
            experiment_name=experiment_name,
            symbol=self.trading_config.trading.symbol,
            start_date=start_date_ny,
            end_date=end_date_ny,
            is_crypto=self.trading_config.trading.is_crypto
        )
        self.experiments = experiments

        best_results, experiments_result = execute_multi_experiment(
            experiment=experiments, 
            debug_mode=self.debug_mode, 
            mlflow_connection=self.mlflow_connection
        )

        self.best_results = best_results
        self.status_bot.experiment_profit_factor = best_results.get('stats_json').get('Profit Factor')
        df_actions = best_results.get('df_actions')


        self.df_actions = df_actions

        self.status_bot.experiment_best_return = best_results.get('best_return')

        if len(df_actions[df_actions['actions'] == 2]) < 2 or  len(df_actions[df_actions['actions'] == 1]) < 2:
            log.info("Best df_actions not have the lot predictions") 
            return float('-inf')


        experiment_return = best_results.get('best_return')
        log.info(f"[EXPERIMENT: {self.experiments.experiment_name}] best_return:  {experiment_return}")
        self.status_bot.experiment_return = experiment_return

        if self.trading_config.calculate_best_stop_loss:
            log.info("Calculate the optimize stop_loss")
            log.info("Start to Calculate The StopLoss")

            best_sl, best_return_sl = optimize_stop_loss_simple(df_actions, st_from=-0.1, st_to=-0.01)
            self.trading_config.trading.stop_loss = best_sl

            self.status_bot.experiment_best_return_stop_loss = best_return_sl
            self.status_bot.experiment_best_stop_loss =  best_sl
            self.status_bot.experiment_profit_factor_sl = best_return_sl
            log.info(f"[EXPERIMENT SL: {self.experiments.experiment_name}] best_return:  {best_return_sl} SL {best_sl}")

            return best_return_sl
        
        return experiment_return
    
    def save_model_mlflow_best(self):

        save_model = False
        if self.trading_config.calculate_best_stop_loss and self.status_bot.experiment_profit_factor_sl > self.min_profit_factor_allowed:
            save_model = True 
            self.best_results['best_return'] = self.status_bot.experiment_profit_factor_sl
        elif not self.trading_config.calculate_best_stop_loss and self.status_bot.experiment_profit_factor > self.min_profit_factor_allowed:
            save_model = True
        
        self.model_name = f"{self.trading_config.trading.symbol}_{self.trading_config.model.tag}"
        if save_model:
            save_best_model_mlflow(
                best_results=self.best_results,
                model_name=self.model_name,
                stage=self.trading_config.model.stage
            )



    def operate(self, force_action :str = None):
        # If not exist order/position run experiments 
        status_bot = self.get_status()

        action = None 
        case = None
        if status_bot.is_registred_model:
            # Get Profit Historical Orders 
            if not status_bot.is_winner:
                best_retrurn = self.run_mlflow_experiments()
                if best_retrurn >= self.min_profit_factor_allowed:
                    log.info("Start Experiment")
                    profit_factor = self.run_mlflow_experiments()
                    if profit_factor > self.min_profit_factor_allowed:
                        log.info("Registring new models")
                        self.save_model_mlflow_best()
                        log.info("Model saved")
            
            log.info("EVAL MODEL")
            action  = eval_mlflow_model_strategy(
                symbol=self.trading_config.trading.symbol, 
                stage=self.trading_config.model.stage, 
                tag=self.trading_config.model.tag, 
                mlflow_connection=self.mlflow_connection, 
                get_action_latest=False
            )

            client_order_id = None
            if action == "open-long" or force_action == "open-long":
                log.info(f"Action: open-long {self.trading_config.trading.symbol}")
                client_order_id = alpaca_open_long(
                                     api_key=self.trading_config.broker.api_key, 
                                     api_secret=self.trading_config.broker.api_secret, 
                                     trading=self.trading_config.trading
                                )
            elif action == "close-long" or force_action == "close-long":
                client_order_id = alpaca_close_long(
                                     api_key=self.trading_config.broker.api_key, 
                                     api_secret=self.trading_config.broker.api_secret, 
                                     symbol=self.trading_config.trading.symbol,
                                     tag=self.trading_config.model.tag
                                )
                
            log.info(f"Action: {action} {self.trading_config.trading.symbol} {client_order_id}")

            case = "case_1"
            
                        
        elif not status_bot.is_registred_model:
            if not status_bot.is_open_order:
                if not status_bot.is_open_positions:
                    profit_factor = self.run_mlflow_experiments()
                    if profit_factor > self.min_profit_factor_allowed:
                        log.info("Registring new models")
                        self.save_model_mlflow_best()
                        log.info("Model saved")

                        if self.trading_config.replicate_latest_open_long_signal:
                            log.info("Replicate latest open-long signal")

                            action  = eval_mlflow_model_strategy(
                                            symbol=self.trading_config.trading.symbol, 
                                            stage=self.trading_config.model.stage, 
                                            tag=self.trading_config.model.tag, 
                                            mlflow_connection=self.mlflow_connection, 
                                            get_action_latest=True
                                        )
                            
                            if action == "open-long" or self.debug_mode==True:
                                 client_order_id = alpaca_open_long(
                                     api_key=self.trading_config.broker.api_key, 
                                     api_secret=self.trading_config.broker.api_secret, 
                                     trading=self.trading_config.trading
                                )
                            case = "case_2"
            

        return action, case



from .stop_loss import *
from .backtesting_custom import *
from .utils import *
from .labels import *
from .indicators import *
from .strategies import *
from .market_data import *
from .strategies.selector import ModelSelector
from .models import *
from .experiments import *
from .bots import *

import os

if not os.path.exists('tmp'):
    os.makedirs('tmp')


from pyttrading import models
from ..utils.logs import log 
import mlflow

def mlflow_save(
        experiment_name: str = "ExperimentName", 
        parent_name: str = "ParentName",
        child_name: str = "ChildName", 
        mlflow_instance=None,
        metrics=None, 
        parameters=None,
        tags=None,
        parent=None,
        experiment_id=None
    ):
    if mlflow_instance is None:
        mlflow_instance = mlflow
    
    log.info(f"Register Experiment {experiment_name}")

    if not experiment_id:
        try:
            experiment_id = mlflow_instance.create_experiment(experiment_name)
        except:
            raise EnvironmentError("Experiment Name not permited, was deleted and is bloqued")

    mlflow_instance.set_experiment(experiment_name)

    if not parent:
        parent = mlflow_instance.start_run(run_name=parent_name)

    with mlflow_instance.start_run(nested=True, run_name=child_name) as child:
        if metrics:
            for key, value in metrics.items():
                key_s = key.replace(" ", '').replace('[%]', '').replace('.', '').replace('&', '').replace('[$]', '').replace('(', '').replace(')', '').replace('#', '')
                if '_' not in key_s:
                    try:
                        mlflow_instance.log_metric(key_s, value)
                    except Exception:
                        pass
        if parameters:
            for key, value in parameters.items():
                key_s = key.replace(" ", '').replace('[%]', '').replace('.', '').replace('&', '').replace('[$]', '').replace('(', '').replace(')', '').replace('#', '')
                mlflow_instance.log_param(key_s, value)
        if tags:
            for key, value in tags.items():
                key_s = key.replace(" ", '').replace('[%]', '').replace('.', '').replace('&', '').replace('[$]', '').replace('(', '').replace(')', '').replace('#', '')
                mlflow_instance.set_tag(key_s, value) 


    try:
        del metrics['_strategy']
        del metrics['_equity_curve']
        del metrics['_trades']
    except:
        pass


    
    return {
        "parent": {
            "run_id": parent.info.run_id,
            "experiment_id": experiment_id,
            "status": parent.info.status,
            "run_name": parent.info.run_name,
            "status": parent.info.status
        },
        "child": {
            "run_id": child.info.run_id,
            "experiment_id": experiment_id,
            "status": child.info.status,
            "run_name": child.info.run_name,
            "status": child.info.status
        }, 
        "parent_experiment": parent,
        "experiment_id": experiment_id,
        "parameters": parameters,
        "tags": tags, 
        "metrics": metrics
    }



def mlflow_read(
        experiment_name :str = None,
        parent_name :str = None
    ):

    client = mlflow.tracking.MlflowClient()
    experiment = client.get_experiment_by_name(name=experiment_name)

    runs = client.search_runs(
        experiment_ids=[experiment.experiment_id],
    )
    
    run_selected = None
    data = None
    for run in  runs:
        info = run.info
        run_name = info.run_name
        if parent_name == run_name:
            run_selected = run
            break
        data = run #TODO verify

    return data



            





import mlflow
from dotenv import dotenv_values
import mlflow
import os


def init_mlflow_connection(mlflow_username :str = None, mlflow_token :str = None, mlflow_host :str = None):
    
    if not mlflow_username:
        config = dotenv_values(".env")  
        mlflow_username = os.getenv('MLFLOW_USERNAME', config.get("MLFLOW_USERNAME"))  
        mlflow_token = os.getenv('MLFLOW_TOKEN', config.get("MLFLOW_TOKEN"))
        mlflow_host = os.getenv('MLFLOW_HOST', config.get("MLFLOW_HOST"))
    
    os.environ['MLFLOW_TRACKING_USERNAME'] = mlflow_username
    os.environ['MLFLOW_TRACKING_PASSWORD'] = mlflow_token
    os.environ['MLFLOW_TRACKING_URI'] = mlflow_host

    mlflow_connection = mlflow
    mlflow_connection.set_tracking_uri(mlflow_host)

    return mlflow_connection


from  .combinations_experiments import generate_experiments_list
from ..models import ExperimentMessage
from ..utils.logs import log
from ..market_data import get_market_data_from_playground
from ..strategies import ModelSelector
from ..experiments import mlflow_save
import json 
import mlflow
import uuid
id_test = str(uuid.uuid4()).replace('-','')[-10:-1]
import traceback

def execute_multi_experiment(experiment :ExperimentMessage, debug_mode :bool=False, mlflow_connection=None):

    log.info(experiment.dict())
    print(experiment.dict())

    experiment_data = generate_experiments_list(experiments=experiment, debug_mode=debug_mode)

    interval_selected = None
    experiments_result = []
    exp_best_return = float('-inf')
    best_results = {}
    parent_experiment = None
    parent_name=experiment_data.get('parent')
    experiment_name=experiment.experiment_name

    try:
        experiment_id = mlflow.get_experiment_by_name(experiment_name).experiment_id
    except Exception:
        experiment_id = mlflow.create_experiment(experiment_name)

    try:
        mlflow.set_experiment(experiment_name)
    except Exception as e:
        assert ConnectionResetError(f"(MLFLOW) ERROR SERVICE: {e}")
    
    with mlflow.start_run(run_name=parent_name) as parent:
        for combination in experiment_data.get('combinations'):
            child_name = combination.get('name')

            print(f">>> COMBINATION:  {combination}")

            with mlflow.start_run(nested=True, run_name=child_name) as child:
                interval_experiment = combination.get('interval')


                for key, value in combination.items():
                    mlflow.set_tag(key, value)


                if not interval_selected or interval_selected != interval_experiment:
                    log.info(f"Search market data with {experiment.symbol} interval {interval_experiment}")
                    data = get_market_data_from_playground(
                        symbols=experiment.symbol, 
                        interval=interval_experiment, 
                        start_date=experiment.start_date,
                        end_date=experiment.end_date,
                        is_crypto=experiment.is_crypto
                    )

                interval_selected = interval_experiment

                model = ModelSelector(
                                model_name=combination.get('strategy'),
                                path_model="tmp",
                                type_model=experiment.tag,
                                configuration=experiment, 
                                df=data,
                                symbol=experiment.symbol,
                                interval=interval_experiment,
                                mlflow=mlflow_connection
                            )

                try:
                    stats_json, best_return, best_return_short, best_parameters, df_actions, params = model.run_experiment_get_backtesting()

                    for key, value in stats_json.items():
                        try:
                            mlflow.log_metric(key, value)
                        except:
                            pass 


                    log.info(f"Experiment: {experiment_name}/{parent_name}/child_name {interval_selected} best_return: {best_return} ")

                    metrics={
                    "best_return": best_return, 
                    **stats_json
                    }

                    for key, value in metrics.items():
                        try:
                            mlflow.log_metric(key, value)
                        except:
                            pass 
                        
                    parameters={
                        "optimized_params": json.dumps(params),
                        "symbol": experiment.symbol,
                        "interval": interval_selected,
                        "is_crypto": experiment.is_crypto
                    }


                    tags = {
                        "method": params.get('method'),
                        "strategy_name": params.get('strategy'),
                        "strategy": params.get('strategy')
                    }
                    response = {
                        "parent": {
                            "run_id": parent.info.run_id,
                            "experiment_id": experiment_id,
                            "status": parent.info.status,
                            "run_name": parent.info.run_name,
                            "status": parent.info.status
                        },
                        "child": {
                            "run_id": child.info.run_id,
                            "experiment_id": experiment_id,
                            "status": child.info.status,
                            "run_name": child.info.run_name,
                            "status": child.info.status
                        }, 
                        "parent_experiment": parent,
                        "experiment_id": experiment_id,
                        "parameters": parameters,
                        "tags": tags, 
                        "metrics": metrics
                    }
        # self.mlflow.set_tag("strategy", self.model_name)
        # self.mlflow.set_tag("method", self.params.get('method'))
        # self.mlflow.log_param("optimized_params", json.dumps(self.params))
        # self.mlflow.log_metric('best_return',  self.params.get('best_return'))

                    if best_return > exp_best_return:
                        exp_best_return = best_return
                        best_results = {
                            "stats_json": stats_json,
                            "best_return": best_return,
                            "best_return_short": best_return_short,
                            "best_parameters": best_parameters,
                            "df_actions": df_actions,
                            "params": params,
                            "experiment_id": experiment_id,
                            "mlflow_save": response
                        }

                    experiments_result.append({
                                "best_return": best_return,
                                "name": combination.get('name'),
                            })
                except Exception as e:
                    log.error(f"EXPERIMENT ERROR: {combination}. Detalles del error:\n{traceback.format_exc()}")
                
    mlflow.end_run()

    return best_results, experiments_result


import json

def save_best_model_mlflow(best_results=None, model_name :str = None, stage :str = "Staging"):

    #TODO fix
    import pyttrading as tt
    from mlflow.tracking import MlflowClient

    log = tt.utils.log

    best_response = best_results.get('mlflow_save')
    
    output = {
        "best_candidate": best_results['best_return'],
        "parms": best_results['params'],
    }

    run_id = best_response.get('child').get('run_id')
    model_uri = f"runs:/{run_id}/model"

    log.info("Saving model")

    metrics = best_response.get('metrics')
    del metrics['_strategy']
    del metrics['_equity_curve']
    del metrics['_trades']

    mlflow_connection = tt.init_mlflow_connection()


    result = mlflow_connection.register_model(
            model_uri=model_uri,
            name= model_name,
            tags={
                "tags": json.dumps(best_response.get('tags')),
                "parameters": json.dumps(best_response.get('parameters')),
                "metrics": json.dumps(best_response.get('metrics'))
            }
        )
    client = MlflowClient()
    
    result_model = client.transition_model_version_stage(
            name=result.name,
            version=result.version,
            stage=stage,
            archive_existing_versions=True
        )  

    profit_factor = round(best_response.get('metrics').get("Profit Factor", -1.001), 2)
    interval = best_response.get('parameters').get('interval')
    client.set_registered_model_tag(result.name, f'{stage}', f"{profit_factor}/{interval}")

    return result_model




from pyttrading import models
from ..utils.logs import log
import uuid
id_test = str(uuid.uuid4()).replace('-','')[-10:-1]


def generate_experiments_list(experiments : models.ExperimentMessage, debug_mode :bool=False):

    combinations = []

    for interval in experiments.intervals:
        for strategy in experiments.strategies_list:
            combinations.append({
                "name": f"{interval}_{strategy}",
                "interval": interval, 
                "strategy": strategy
            })


    if debug_mode:
        if len(combinations) > 1:
            combinations = [combinations[0], combinations[1]]
        else:
            combinations = [combinations[0]]


            
    experiments_all = {
        experiments.experiment_name: {
            f"{experiments.start_date}-{experiments.end_date}": combinations
        }
    }

    experiments_data = {
        "combinations": combinations, 
        "parent": f"{experiments.start_date}-{experiments.end_date}",
        "experiment": experiments.experiment_name,
        "all": experiments_all
    }

    return experiments_data


from pydantic import BaseModel
from config import Config 
config = Config()
from .. import intervals_list, strategies_list
from datetime import datetime, timedelta
import pytz
from enum import Enum


# 'America/New_York'
def get_datetime_now_trade(timezone :str = 'America/New_York', delay_minutes = 15, days_ago = 100):
    
    ny_timezone = pytz.timezone(timezone)
    now_ny = datetime.now(ny_timezone)
    start_date_ny = now_ny - timedelta(days=days_ago)
    start_date_ny = start_date_ny.replace(hour=0, minute=0, second=0, microsecond=0)
    start_date_ny = start_date_ny.strftime('%m/%d/%Y')
    end_date_ny = now_ny - timedelta(minutes=delay_minutes)
    end_date_ny = end_date_ny.strftime('%m/%d/%Y %H:%M')
 
    return start_date_ny, end_date_ny, now_ny

start_date, end_date, now_ny =  get_datetime_now_trade(delay_minutes=int(config.delay_minutes))

class StrategyParams(BaseModel):

    symbol: str = "TNA"
    interval: str = "1h"
    is_crypto: bool = False

class Trade(StrategyParams):
    start_date: str = str(start_date)
    end_date: str = str(end_date)
    time_amount: int = 10
    indicators: list = [
                        "boll_ub",
                        "boll_lb",
                        "close_10_sma",
                        "close_12_ema",
                        "close_16_ema"
                        ]


class Experiment(Trade):
    
    version :str = "2.0.0"
    tag :str='algo'
    experiment_name: str = f"{Trade().symbol}_{tag}_{version}"

class ExperimentMessage(Experiment):

    intervals: list = intervals_list
    strategies_list: list = strategies_list
    start_date: str = '9/1/2023'
    end_date: str = '1/5/2024'



class ExperimentOne(Experiment):
    tag: str = "rest"
    interval: str ='1h'
    start_date: str = '9/1/2023'
    end_date: str = '1/5/2024'
    strategy: str =  'rsi'



class StageEnum(str, Enum):
    Production = "Production"
    Staging = "Staging"
    Archived = "Archived"


from .ema import Strategy as ema
from .rsi import Strategy as rsi
from .macd import Strategy as macd
from .sma import Strategy as sma
from .dummy import Strategy as dummy
from .selector import ModelSelector

strategies_list = ['ema', 'rsi', 'macd', 'sma', 'dummy']

intervals_list = [
        '1h',
        # '2h'
        # '4h',
        # '6h',
        # '8h',
    ]